@@ -1,3 +1,4 @@
+import ast
 import re
 import sys
 import itertools
@@ -7,6 +8,10 @@ import functools
 import collections
 import pickle
 import textwrap
+try:
+    from ast import TryExcept
+except ImportError:  # Python 3
+    from ast import Try as TryExcept
 
 from .astutil import load
 from .astutil import store
@@ -21,6 +26,7 @@ from .astutil import Symbol
 from .astutil import Builtin
 from .astutil import Static
 from .astutil import TokenRef
+from .astutil import Node
 
 from .codegen import TemplateCodeGenerator
 from .codegen import template
@@ -35,6 +41,11 @@ from .nodes import Substitution
 from .nodes import Assignment
 from .nodes import Module
 from .nodes import Context
+from .nodes import Is
+from .nodes import IsNot
+from .nodes import Equals
+from .nodes import Logical
+from .nodes import And
 
 from .tokenize import Token
 from .config import DEBUG_MODE
@@ -49,13 +60,12 @@ from .utils import native_string
 from .utils import byte_string
 from .utils import string_type
 from .utils import unicode_string
-from .utils import version
-from .utils import ast
 from .utils import safe_native
 from .utils import builtins
 from .utils import decode_htmlentities
+from .utils import join
 
-if version >= (3, 0, 0):
+if sys.version_info >= (3,):
     long = int
 
 log = logging.getLogger('chameleon.compiler')
@@ -133,10 +143,6 @@ def eval_token(token):
     )
 
 
-emit_node = template(is_func=True, func_args=('node',), source=r"""
-    __append(node)""")
-
-
 emit_node_if_non_trivial = template(is_func=True, func_args=('node',),
                                     source=r"""
     if node is not None:
@@ -270,6 +276,22 @@ emit_func_convert_and_escape = template(
         return target""")
 
 
+class EmitText(Node):
+    """Append text to output."""
+
+    _fields = "s",
+
+
+class Scope(Node):
+    """"Set a local output scope."""
+
+    _fields = "body", "append", "stream"
+
+    body = None
+    append = None
+    stream = None
+
+
 class Interpolator(object):
     braces_required_regex = re.compile(
         r'(\$)?\$({(?P<expression>.*)})',
@@ -805,7 +827,10 @@ class ExpressionTransform(object):
         return stmts
 
     def visit_Value(self, node, target):
-        engine = self.engine_factory()
+        engine = self.engine_factory(
+            default=node.default,
+            default_marker=node.default_marker
+        )
         compiler = engine.parse(node.value)
         return compiler.assign_value(target)
 
@@ -813,7 +838,10 @@ class ExpressionTransform(object):
         return self.translate(node.expression, target)
 
     def visit_Substitution(self, node, target):
-        engine = self.engine_factory(default=node.default)
+        engine = self.engine_factory(
+            default=node.default,
+            default_marker=node.default_marker
+        )
         compiler = engine.parse(node.value, char_escape=node.char_escape)
         return compiler.assign_text(target)
 
@@ -821,22 +849,23 @@ class ExpressionTransform(object):
         return self.translate(node.value, target) + \
                template("TARGET = not TARGET", TARGET=target)
 
-    def visit_Identity(self, node, target):
-        expression = self.translate(node.expression, "__expression")
-        value = self.translate(node.value, "__value")
+    def visit_BinOp(self, node, target):
+        expression = self.translate(node.left, "__expression")
+        value = self.translate(node.right, "__value")
 
+        op = {
+            Is: "is",
+            IsNot: "is not",
+            Equals: "==",
+        }[node.op]
         return expression + value + \
-               template("TARGET = __expression is __value", TARGET=target)
-
-    def visit_Equality(self, node, target):
-        expression = self.translate(node.expression, "__expression")
-        value = self.translate(node.value, "__value")
-
-        return expression + value + \
-               template("TARGET = __expression == __value", TARGET=target)
+               template("TARGET = __expression %s __value" % op, TARGET=target)
 
     def visit_Boolean(self, node, target):
-        engine = self.engine_factory()
+        engine = self.engine_factory(
+            default=node.default,
+            default_marker=node.default_marker,
+        )
         compiler = engine.parse(node.value)
         return compiler.assign_bool(target, node.s)
 
@@ -846,9 +875,13 @@ class ExpressionTransform(object):
             engine = self.engine_factory(
                 char_escape=expr.char_escape,
                 default=expr.default,
-                )
+                default_marker=expr.default_marker
+            )
         elif isinstance(expr, Value):
-            engine = self.engine_factory()
+            engine = self.engine_factory(
+                default=expr.default,
+                default_marker=expr.default_marker
+            )
         else:
             raise RuntimeError("Bad value: %r." % node.value)
 
@@ -949,7 +982,26 @@ class Compiler(object):
             module = ast.Module([])
             module.body += self.visit(node)
             ast.fix_missing_locations(module)
-            generator = TemplateCodeGenerator(module, source)
+
+            class Generator(TemplateCodeGenerator):
+                scopes = [Scope()]
+
+                def visit_EmitText(self, node):
+                    append = load(self.scopes[-1].append or "__append")
+                    for node in template("append(s)", append=append, s=ast.Str(s=node.s)):
+                        self.visit(node)
+
+                def visit_Scope(self, node):
+                    self.scopes.append(node)
+                    body = list(node.body)
+                    swap(body, load(node.append), "__append")
+                    if node.stream:
+                        swap(body, load(node.stream), "__stream")
+                    for node in body:
+                        self.visit(node)
+                    self.scopes.pop()
+
+            generator = Generator(module, source)
             tokens = [
                 Token(source[pos:pos + length], pos, source)
                 for pos, length in generator.tokens
@@ -976,7 +1028,15 @@ class Compiler(object):
         kind = type(node).__name__
         visitor = getattr(self, "visit_%s" % kind)
         iterator = visitor(node)
-        return list(iterator)
+        result = []
+        for key, group in itertools.groupby(iterator, lambda node: node.__class__):
+            nodes = list(group)
+            if key is EmitText:
+                text = join(node.s for node in nodes)
+                nodes = [EmitText(text)]
+            result.extend(nodes)
+        return result
+
 
     def visit_Sequence(self, node):
         for item in node.items:
@@ -1000,7 +1060,7 @@ class Compiler(object):
         body += template("import re")
         body += template("import functools")
         body += template("from itertools import chain as __chain")
-        if version < (3, 0, 0):
+        if sys.version_info < (3,):
             body += template("from sys import exc_clear as __exc_clear")
         else:
             body += template("from sys import intern")
@@ -1101,7 +1161,7 @@ class Compiler(object):
 
         # Wrap visited nodes in try-except error handler.
         body += [
-            ast.TryExcept(
+            TryExcept(
                 body=nodes,
                 handlers=[ast.ExceptHandler(body=exc_handler)]
             )
@@ -1127,7 +1187,7 @@ class Compiler(object):
         yield function
 
     def visit_Text(self, node):
-        return emit_node(ast.Str(s=node.value))
+        yield EmitText(node.value)
 
     def visit_Domain(self, node):
         backup = "__previous_i18n_domain_%s" % mangle(id(node))
@@ -1161,7 +1221,7 @@ class Compiler(object):
             key=ast.Str(s=node.name),
             )
 
-        body += [ast.TryExcept(
+        body += [TryExcept(
             body=self.visit(node.node),
             handlers=[ast.ExceptHandler(
                 type=ast.Tuple(elts=[Builtin("Exception")], ctx=ast.Load()),
@@ -1256,7 +1316,7 @@ class Compiler(object):
         for stmt in self.visit(node.node):
             yield stmt
 
-        for assignment in node.assignments:
+        for assignment in reversed(node.assignments):
             if assignment.local:
                 for stmt in self._leave_assignment(assignment.names):
                     yield stmt
@@ -1269,22 +1329,44 @@ class Compiler(object):
 
     def visit_Condition(self, node):
         target = "__condition"
-        assignment = self._engine(node.expression, target)
 
-        assert assignment
+        def step(expressions, body, condition):
+            for i, expression in enumerate(reversed(expressions)):
+                stmts = evaluate(expression, body)
+                if i > 0:
+                    stmts.append(
+                        ast.If(
+                            ast.Compare(
+                                left=load(target),
+                                ops=[ast.Is()],
+                                comparators=[load(str(condition))]
+                            ),
+                            body,
+                            None
+                        )
+                    )
+                body = stmts
+            return body
 
-        for stmt in assignment:
-            yield stmt
+        def evaluate(node, body=None):
+            if isinstance(node, Logical):
+                condition = isinstance(node, And)
+                return step(node.expressions, body, condition)
 
-        body = self.visit(node.node) or [ast.Pass()]
+            return self._engine(node, target)
 
+        body = evaluate(node.expression)
         orelse = getattr(node, "orelse", None)
-        if orelse is not None:
-            orelse = self.visit(orelse)
 
-        test = load(target)
+        body.append(
+            ast.If(
+                test=load(target),
+                body=self.visit(node.node) or [ast.Pass()],
+                orelse=self.visit(orelse) if orelse else None,
+            )
+        )
 
-        yield ast.If(test=test, body=body, orelse=orelse)
+        return body
 
     def visit_Translate(self, node):
         """Translation.
@@ -1308,9 +1390,7 @@ class Compiler(object):
 
         # Visit body to generate the message body
         code = self.visit(node.node)
-        swap(code, load(append), "__append")
-        swap(code, load(stream), "__stream")
-        body += code
+        body.append(Scope(code, append, stream))
 
         # Reduce white space and assign as message id
         msgid = identifier("msgid", id(node))
@@ -1371,23 +1451,16 @@ class Compiler(object):
                 node.prefix, node.name, line, column))
 
         if node.attributes:
-            for stmt in emit_node(ast.Str(s=node.prefix + node.name)):
-                yield stmt
-
+            yield EmitText(node.prefix + node.name)
             for stmt in self.visit(node.attributes):
                 yield stmt
 
-            for stmt in emit_node(ast.Str(s=node.suffix)):
-                yield stmt
+            yield EmitText(node.suffix)
         else:
-            for stmt in emit_node(
-                ast.Str(s=node.prefix + node.name + node.suffix)):
-                yield stmt
+            yield EmitText(node.prefix + node.name + node.suffix)
 
     def visit_End(self, node):
-        for stmt in emit_node(ast.Str(
-            s=node.prefix + node.name + node.space + node.suffix)):
-            yield stmt
+        yield EmitText(node.prefix + node.name + node.space + node.suffix)
 
     def visit_Attribute(self, node):
         attr_format = (node.space + node.name + node.eq +
@@ -1416,7 +1489,7 @@ class Compiler(object):
                     "if C: __append(S)", C=filter_condition, S=ast.Str(s=s)
                 )
             else:
-                return template("__append(S)", S=ast.Str(s=s))
+                return [EmitText(s)]
 
         target = identifier("attr", node.name)
         body = self._engine(node.expression, store(target))
@@ -1466,13 +1539,13 @@ class Compiler(object):
         body = []
 
         for expression in node.expressions:
-            name = identifier("cache", id(expression))
-            target = store(name)
-
             # Skip re-evaluation
             if self._expression_cache.get(expression):
                 continue
 
+            name = identifier("cache", id(expression))
+            target = store(name)
+
             body += self._engine(expression, target)
             self._expression_cache[expression] = target
 
@@ -1484,13 +1557,10 @@ class Compiler(object):
         body = []
 
         for expression in node.expressions:
+            assert self._expression_cache.get(expression) is not None
             name = identifier("cache", id(expression))
             target = store(name)
-
-            if not self._expression_cache.get(expression):
-               continue
-
-            body.append(ast.Assign([target], load("None")))
+            body += self._engine(node.value, target)
 
         body += self.visit(node.node)
 
@@ -1547,8 +1617,7 @@ class Compiler(object):
 
         # generate code
         code = self.visit(node.node)
-        swap(code, load(append), "__append")
-        body += code
+        body.append(Scope(code, append))
 
         # output msgid
         text = Text('${%s}' % node.name)
@@ -1609,7 +1678,7 @@ class Compiler(object):
             if node.extend:
                 append = template("_slots.appendleft(NAME)", NAME=fun)
 
-                assignment = [ast.TryExcept(
+                assignment = [TryExcept(
                     body=template("_slots = getitem(KEY)", KEY=key),
                     handlers=[ast.ExceptHandler(body=assignment)],
                     orelse=append,
